# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .show_base import ShowBase as ShowBase
from .image_object import ImageObject as ImageObject
from .track_object import TrackObject as TrackObject
from .artist_object import ArtistObject as ArtistObject
from .author_object import AuthorObject as AuthorObject
from .audiobook_base import AudiobookBase as AudiobookBase
from .episode_object import EpisodeObject as EpisodeObject
from .narrator_object import NarratorObject as NarratorObject
from .copyright_object import CopyrightObject as CopyrightObject
from .followers_object import FollowersObject as FollowersObject
from .external_id_object import ExternalIDObject as ExternalIDObject
from .external_url_object import ExternalURLObject as ExternalURLObject
from .linked_track_object import LinkedTrackObject as LinkedTrackObject
from .resume_point_object import ResumePointObject as ResumePointObject
from .playlist_user_object import PlaylistUserObject as PlaylistUserObject
from .playlist_track_object import PlaylistTrackObject as PlaylistTrackObject
from .paging_playlist_object import PagingPlaylistObject as PagingPlaylistObject
from .simplified_track_object import SimplifiedTrackObject as SimplifiedTrackObject
from .album_restriction_object import AlbumRestrictionObject as AlbumRestrictionObject
from .simplified_artist_object import SimplifiedArtistObject as SimplifiedArtistObject
from .track_restriction_object import TrackRestrictionObject as TrackRestrictionObject
from .simplified_episode_object import SimplifiedEpisodeObject as SimplifiedEpisodeObject
from .chapter_restriction_object import ChapterRestrictionObject as ChapterRestrictionObject
from .episode_restriction_object import EpisodeRestrictionObject as EpisodeRestrictionObject
from .playlist_tracks_ref_object import PlaylistTracksRefObject as PlaylistTracksRefObject
from .simplified_playlist_object import SimplifiedPlaylistObject as SimplifiedPlaylistObject



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["TrackRestrictionObject"]


class TrackRestrictionObject(BaseModel):
    reason: Optional[str] = None
    """The reason for the restriction. Supported values:

    - `market` - The content item is not available in the given market.
    - `product` - The content item is not available for the user's subscription
      type.
    - `explicit` - The content item is explicit and the user's account is set to not
      play explicit content.

    Additional reasons may be added in the future. **Note**: If you use this field,
    make sure that your application safely handles unknown values.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._oauth2 import OAuth2ClientCredentials, make_oauth2
from ._version import __version__
from .resources import (
    shows,
    albums,
    search,
    tracks,
    artists,
    markets,
    chapters,
    episodes,
    audiobooks,
    audio_analysis,
    audio_features,
    recommendations,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.me import me
from .resources.users import users
from .resources.browse import browse
from .resources.playlists import playlists

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Spotted", "AsyncSpotted", "Client", "AsyncClient"]


class Spotted(SyncAPIClient):
    albums: albums.AlbumsResource
    artists: artists.ArtistsResource
    shows: shows.ShowsResource
    episodes: episodes.EpisodesResource
    audiobooks: audiobooks.AudiobooksResource
    me: me.MeResource
    chapters: chapters.ChaptersResource
    tracks: tracks.TracksResource
    search: search.SearchResource
    playlists: playlists.PlaylistsResource
    users: users.UsersResource
    browse: browse.BrowseResource
    audio_features: audio_features.AudioFeaturesResource
    audio_analysis: audio_analysis.AudioAnalysisResource
    recommendations: recommendations.RecommendationsResource
    markets: markets.MarketsResource
    with_raw_response: SpottedWithRawResponse
    with_streaming_response: SpottedWithStreamedResponse

    # client options
    client_id: str | None
    client_secret: str | None
    access_token: str | None

    def __init__(
        self,
        *,
        client_id: str | None = None,
        client_secret: str | None = None,
        access_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Spotted client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `client_id` from `SPOTIFY_CLIENT_ID`
        - `client_secret` from `SPOTIFY_CLIENT_SECRET`
        - `access_token` from `SPOTIFY_ACCESS_TOKEN`
        """
        if client_id is None:
            client_id = os.environ.get("SPOTIFY_CLIENT_ID")
        self.client_id = client_id

        if client_secret is None:
            client_secret = os.environ.get("SPOTIFY_CLIENT_SECRET")
        self.client_secret = client_secret

        if access_token is None:
            access_token = os.environ.get("SPOTIFY_ACCESS_TOKEN")
        self.access_token = access_token

        if base_url is None:
            base_url = os.environ.get("SPOTTED_BASE_URL")
        if base_url is None:
            base_url = f"https://api.spotify.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.albums = albums.AlbumsResource(self)
        self.artists = artists.ArtistsResource(self)
        self.shows = shows.ShowsResource(self)
        self.episodes = episodes.EpisodesResource(self)
        self.audiobooks = audiobooks.AudiobooksResource(self)
        self.me = me.MeResource(self)
        self.chapters = chapters.ChaptersResource(self)
        self.tracks = tracks.TracksResource(self)
        self.search = search.SearchResource(self)
        self.playlists = playlists.PlaylistsResource(self)
        self.users = users.UsersResource(self)
        self.browse = browse.BrowseResource(self)
        self.audio_features = audio_features.AudioFeaturesResource(self)
        self.audio_analysis = audio_analysis.AudioAnalysisResource(self)
        self.recommendations = recommendations.RecommendationsResource(self)
        self.markets = markets.MarketsResource(self)
        self.with_raw_response = SpottedWithRawResponse(self)
        self.with_streaming_response = SpottedWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        access_token = self.access_token
        if access_token is None:
            return {}
        return {"Authorization": f"Bearer {access_token}"}

    @property
    @override
    def custom_auth(self) -> httpx.Auth | None:
        if self.client_id and self.client_secret:
            return make_oauth2(
                client_id=self.client_id,
                client_secret=self.client_secret,
                token_url=self._prepare_url("https://accounts.spotify.com/api/token"),
                header="Authorization",
            )
        return None

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    @override
    def _should_retry(self, response: httpx.Response) -> bool:
        # Retry on 401 if we are using OAuth2 and the token might be expired
        if response.status_code == 401 and isinstance(self.custom_auth, OAuth2ClientCredentials):
            if self.custom_auth.token_is_expired():
                self.custom_auth.invalidate_token()
                return True
        return super()._should_retry(response)

    def copy(
        self,
        *,
        client_id: str | None = None,
        client_secret: str | None = None,
        access_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            client_id=client_id or self.client_id,
            client_secret=client_secret or self.client_secret,
            access_token=access_token or self.access_token,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncSpotted(AsyncAPIClient):
    albums: albums.AsyncAlbumsResource
    artists: artists.AsyncArtistsResource
    shows: shows.AsyncShowsResource
    episodes: episodes.AsyncEpisodesResource
    audiobooks: audiobooks.AsyncAudiobooksResource
    me: me.AsyncMeResource
    chapters: chapters.AsyncChaptersResource
    tracks: tracks.AsyncTracksResource
    search: search.AsyncSearchResource
    playlists: playlists.AsyncPlaylistsResource
    users: users.AsyncUsersResource
    browse: browse.AsyncBrowseResource
    audio_features: audio_features.AsyncAudioFeaturesResource
    audio_analysis: audio_analysis.AsyncAudioAnalysisResource
    recommendations: recommendations.AsyncRecommendationsResource
    markets: markets.AsyncMarketsResource
    with_raw_response: AsyncSpottedWithRawResponse
    with_streaming_response: AsyncSpottedWithStreamedResponse

    # client options
    client_id: str | None
    client_secret: str | None
    access_token: str | None

    def __init__(
        self,
        *,
        client_id: str | None = None,
        client_secret: str | None = None,
        access_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncSpotted client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `client_id` from `SPOTIFY_CLIENT_ID`
        - `client_secret` from `SPOTIFY_CLIENT_SECRET`
        - `access_token` from `SPOTIFY_ACCESS_TOKEN`
        """
        if client_id is None:
            client_id = os.environ.get("SPOTIFY_CLIENT_ID")
        self.client_id = client_id

        if client_secret is None:
            client_secret = os.environ.get("SPOTIFY_CLIENT_SECRET")
        self.client_secret = client_secret

        if access_token is None:
            access_token = os.environ.get("SPOTIFY_ACCESS_TOKEN")
        self.access_token = access_token

        if base_url is None:
            base_url = os.environ.get("SPOTTED_BASE_URL")
        if base_url is None:
            base_url = f"https://api.spotify.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.albums = albums.AsyncAlbumsResource(self)
        self.artists = artists.AsyncArtistsResource(self)
        self.shows = shows.AsyncShowsResource(self)
        self.episodes = episodes.AsyncEpisodesResource(self)
        self.audiobooks = audiobooks.AsyncAudiobooksResource(self)
        self.me = me.AsyncMeResource(self)
        self.chapters = chapters.AsyncChaptersResource(self)
        self.tracks = tracks.AsyncTracksResource(self)
        self.search = search.AsyncSearchResource(self)
        self.playlists = playlists.AsyncPlaylistsResource(self)
        self.users = users.AsyncUsersResource(self)
        self.browse = browse.AsyncBrowseResource(self)
        self.audio_features = audio_features.AsyncAudioFeaturesResource(self)
        self.audio_analysis = audio_analysis.AsyncAudioAnalysisResource(self)
        self.recommendations = recommendations.AsyncRecommendationsResource(self)
        self.markets = markets.AsyncMarketsResource(self)
        self.with_raw_response = AsyncSpottedWithRawResponse(self)
        self.with_streaming_response = AsyncSpottedWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        access_token = self.access_token
        if access_token is None:
            return {}
        return {"Authorization": f"Bearer {access_token}"}

    @property
    @override
    def custom_auth(self) -> httpx.Auth | None:
        if self.client_id and self.client_secret:
            return make_oauth2(
                client_id=self.client_id,
                client_secret=self.client_secret,
                token_url=self._prepare_url("https://accounts.spotify.com/api/token"),
                header="Authorization",
            )
        return None

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    @override
    def _should_retry(self, response: httpx.Response) -> bool:
        # Retry on 401 if we are using OAuth2 and the token might be expired
        if response.status_code == 401 and isinstance(self.custom_auth, OAuth2ClientCredentials):
            if self.custom_auth.token_is_expired():
                self.custom_auth.invalidate_token()
                return True
        return super()._should_retry(response)

    def copy(
        self,
        *,
        client_id: str | None = None,
        client_secret: str | None = None,
        access_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            client_id=client_id or self.client_id,
            client_secret=client_secret or self.client_secret,
            access_token=access_token or self.access_token,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class SpottedWithRawResponse:
    def __init__(self, client: Spotted) -> None:
        self.albums = albums.AlbumsResourceWithRawResponse(client.albums)
        self.artists = artists.ArtistsResourceWithRawResponse(client.artists)
        self.shows = shows.ShowsResourceWithRawResponse(client.shows)
        self.episodes = episodes.EpisodesResourceWithRawResponse(client.episodes)
        self.audiobooks = audiobooks.AudiobooksResourceWithRawResponse(client.audiobooks)
        self.me = me.MeResourceWithRawResponse(client.me)
        self.chapters = chapters.ChaptersResourceWithRawResponse(client.chapters)
        self.tracks = tracks.TracksResourceWithRawResponse(client.tracks)
        self.search = search.SearchResourceWithRawResponse(client.search)
        self.playlists = playlists.PlaylistsResourceWithRawResponse(client.playlists)
        self.users = users.UsersResourceWithRawResponse(client.users)
        self.browse = browse.BrowseResourceWithRawResponse(client.browse)
        self.audio_features = audio_features.AudioFeaturesResourceWithRawResponse(client.audio_features)
        self.audio_analysis = audio_analysis.AudioAnalysisResourceWithRawResponse(client.audio_analysis)
        self.recommendations = recommendations.RecommendationsResourceWithRawResponse(client.recommendations)
        self.markets = markets.MarketsResourceWithRawResponse(client.markets)


class AsyncSpottedWithRawResponse:
    def __init__(self, client: AsyncSpotted) -> None:
        self.albums = albums.AsyncAlbumsResourceWithRawResponse(client.albums)
        self.artists = artists.AsyncArtistsResourceWithRawResponse(client.artists)
        self.shows = shows.AsyncShowsResourceWithRawResponse(client.shows)
        self.episodes = episodes.AsyncEpisodesResourceWithRawResponse(client.episodes)
        self.audiobooks = audiobooks.AsyncAudiobooksResourceWithRawResponse(client.audiobooks)
        self.me = me.AsyncMeResourceWithRawResponse(client.me)
        self.chapters = chapters.AsyncChaptersResourceWithRawResponse(client.chapters)
        self.tracks = tracks.AsyncTracksResourceWithRawResponse(client.tracks)
        self.search = search.AsyncSearchResourceWithRawResponse(client.search)
        self.playlists = playlists.AsyncPlaylistsResourceWithRawResponse(client.playlists)
        self.users = users.AsyncUsersResourceWithRawResponse(client.users)
        self.browse = browse.AsyncBrowseResourceWithRawResponse(client.browse)
        self.audio_features = audio_features.AsyncAudioFeaturesResourceWithRawResponse(client.audio_features)
        self.audio_analysis = audio_analysis.AsyncAudioAnalysisResourceWithRawResponse(client.audio_analysis)
        self.recommendations = recommendations.AsyncRecommendationsResourceWithRawResponse(client.recommendations)
        self.markets = markets.AsyncMarketsResourceWithRawResponse(client.markets)


class SpottedWithStreamedResponse:
    def __init__(self, client: Spotted) -> None:
        self.albums = albums.AlbumsResourceWithStreamingResponse(client.albums)
        self.artists = artists.ArtistsResourceWithStreamingResponse(client.artists)
        self.shows = shows.ShowsResourceWithStreamingResponse(client.shows)
        self.episodes = episodes.EpisodesResourceWithStreamingResponse(client.episodes)
        self.audiobooks = audiobooks.AudiobooksResourceWithStreamingResponse(client.audiobooks)
        self.me = me.MeResourceWithStreamingResponse(client.me)
        self.chapters = chapters.ChaptersResourceWithStreamingResponse(client.chapters)
        self.tracks = tracks.TracksResourceWithStreamingResponse(client.tracks)
        self.search = search.SearchResourceWithStreamingResponse(client.search)
        self.playlists = playlists.PlaylistsResourceWithStreamingResponse(client.playlists)
        self.users = users.UsersResourceWithStreamingResponse(client.users)
        self.browse = browse.BrowseResourceWithStreamingResponse(client.browse)
        self.audio_features = audio_features.AudioFeaturesResourceWithStreamingResponse(client.audio_features)
        self.audio_analysis = audio_analysis.AudioAnalysisResourceWithStreamingResponse(client.audio_analysis)
        self.recommendations = recommendations.RecommendationsResourceWithStreamingResponse(client.recommendations)
        self.markets = markets.MarketsResourceWithStreamingResponse(client.markets)


class AsyncSpottedWithStreamedResponse:
    def __init__(self, client: AsyncSpotted) -> None:
        self.albums = albums.AsyncAlbumsResourceWithStreamingResponse(client.albums)
        self.artists = artists.AsyncArtistsResourceWithStreamingResponse(client.artists)
        self.shows = shows.AsyncShowsResourceWithStreamingResponse(client.shows)
        self.episodes = episodes.AsyncEpisodesResourceWithStreamingResponse(client.episodes)
        self.audiobooks = audiobooks.AsyncAudiobooksResourceWithStreamingResponse(client.audiobooks)
        self.me = me.AsyncMeResourceWithStreamingResponse(client.me)
        self.chapters = chapters.AsyncChaptersResourceWithStreamingResponse(client.chapters)
        self.tracks = tracks.AsyncTracksResourceWithStreamingResponse(client.tracks)
        self.search = search.AsyncSearchResourceWithStreamingResponse(client.search)
        self.playlists = playlists.AsyncPlaylistsResourceWithStreamingResponse(client.playlists)
        self.users = users.AsyncUsersResourceWithStreamingResponse(client.users)
        self.browse = browse.AsyncBrowseResourceWithStreamingResponse(client.browse)
        self.audio_features = audio_features.AsyncAudioFeaturesResourceWithStreamingResponse(client.audio_features)
        self.audio_analysis = audio_analysis.AsyncAudioAnalysisResourceWithStreamingResponse(client.audio_analysis)
        self.recommendations = recommendations.AsyncRecommendationsResourceWithStreamingResponse(client.recommendations)
        self.markets = markets.AsyncMarketsResourceWithStreamingResponse(client.markets)


Client = Spotted

AsyncClient = AsyncSpotted



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from ..shared.image_object import ImageObject

__all__ = ["ImageListResponse"]

ImageListResponse: TypeAlias = List[ImageObject]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["CopyrightObject"]


class CopyrightObject(BaseModel):
    text: Optional[str] = None
    """The copyright text for this content."""

    type: Optional[str] = None
    """
    The type of copyright: `C` = the copyright, `P` = the sound recording
    (performance) copyright.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["TimeIntervalObject"]


class TimeIntervalObject(BaseModel):
    confidence: Optional[float] = None
    """The confidence, from 0.0 to 1.0, of the reliability of the interval."""

    duration: Optional[float] = None
    """The duration (in seconds) of the time interval."""

    start: Optional[float] = None
    """The starting point (in seconds) of the time interval."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types.me import (
    FollowingCheckResponse,
    FollowingBulkRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFollowing:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_retrieve(self, client: Spotted) -> None:
        following = client.me.following.bulk_retrieve(
            type="artist",
        )
        assert_matches_type(FollowingBulkRetrieveResponse, following, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_retrieve_with_all_params(self, client: Spotted) -> None:
        following = client.me.following.bulk_retrieve(
            type="artist",
            after="0I2XqVXqHScXjHhk6AYYRe",
            limit=10,
        )
        assert_matches_type(FollowingBulkRetrieveResponse, following, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_bulk_retrieve(self, client: Spotted) -> None:
        response = client.me.following.with_raw_response.bulk_retrieve(
            type="artist",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        following = response.parse()
        assert_matches_type(FollowingBulkRetrieveResponse, following, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_bulk_retrieve(self, client: Spotted) -> None:
        with client.me.following.with_streaming_response.bulk_retrieve(
            type="artist",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            following = response.parse()
            assert_matches_type(FollowingBulkRetrieveResponse, following, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_check(self, client: Spotted) -> None:
        following = client.me.following.check(
            ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            type="artist",
        )
        assert_matches_type(FollowingCheckResponse, following, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_check(self, client: Spotted) -> None:
        response = client.me.following.with_raw_response.check(
            ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            type="artist",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        following = response.parse()
        assert_matches_type(FollowingCheckResponse, following, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_check(self, client: Spotted) -> None:
        with client.me.following.with_streaming_response.check(
            ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            type="artist",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            following = response.parse()
            assert_matches_type(FollowingCheckResponse, following, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_follow(self, client: Spotted) -> None:
        following = client.me.following.follow(
            ids=["string"],
        )
        assert following is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_follow(self, client: Spotted) -> None:
        response = client.me.following.with_raw_response.follow(
            ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        following = response.parse()
        assert following is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_follow(self, client: Spotted) -> None:
        with client.me.following.with_streaming_response.follow(
            ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            following = response.parse()
            assert following is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_unfollow(self, client: Spotted) -> None:
        following = client.me.following.unfollow()
        assert following is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_unfollow_with_all_params(self, client: Spotted) -> None:
        following = client.me.following.unfollow(
            ids=["string"],
        )
        assert following is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_unfollow(self, client: Spotted) -> None:
        response = client.me.following.with_raw_response.unfollow()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        following = response.parse()
        assert following is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_unfollow(self, client: Spotted) -> None:
        with client.me.following.with_streaming_response.unfollow() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            following = response.parse()
            assert following is None

        assert cast(Any, response.is_closed) is True


class TestAsyncFollowing:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        following = await async_client.me.following.bulk_retrieve(
            type="artist",
        )
        assert_matches_type(FollowingBulkRetrieveResponse, following, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_retrieve_with_all_params(self, async_client: AsyncSpotted) -> None:
        following = await async_client.me.following.bulk_retrieve(
            type="artist",
            after="0I2XqVXqHScXjHhk6AYYRe",
            limit=10,
        )
        assert_matches_type(FollowingBulkRetrieveResponse, following, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.following.with_raw_response.bulk_retrieve(
            type="artist",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        following = await response.parse()
        assert_matches_type(FollowingBulkRetrieveResponse, following, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.following.with_streaming_response.bulk_retrieve(
            type="artist",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            following = await response.parse()
            assert_matches_type(FollowingBulkRetrieveResponse, following, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_check(self, async_client: AsyncSpotted) -> None:
        following = await async_client.me.following.check(
            ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            type="artist",
        )
        assert_matches_type(FollowingCheckResponse, following, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_check(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.following.with_raw_response.check(
            ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            type="artist",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        following = await response.parse()
        assert_matches_type(FollowingCheckResponse, following, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_check(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.following.with_streaming_response.check(
            ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            type="artist",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            following = await response.parse()
            assert_matches_type(FollowingCheckResponse, following, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_follow(self, async_client: AsyncSpotted) -> None:
        following = await async_client.me.following.follow(
            ids=["string"],
        )
        assert following is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_follow(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.following.with_raw_response.follow(
            ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        following = await response.parse()
        assert following is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_follow(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.following.with_streaming_response.follow(
            ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            following = await response.parse()
            assert following is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_unfollow(self, async_client: AsyncSpotted) -> None:
        following = await async_client.me.following.unfollow()
        assert following is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_unfollow_with_all_params(self, async_client: AsyncSpotted) -> None:
        following = await async_client.me.following.unfollow(
            ids=["string"],
        )
        assert following is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_unfollow(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.following.with_raw_response.unfollow()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        following = await response.parse()
        assert following is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_unfollow(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.following.with_streaming_response.unfollow() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            following = await response.parse()
            assert following is None

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Annotated, TypeAlias

from ...._utils import PropertyInfo
from ...._models import BaseModel
from ...shared.track_object import TrackObject
from ...shared.episode_object import EpisodeObject

__all__ = ["QueueGetResponse", "CurrentlyPlaying", "Queue"]

CurrentlyPlaying: TypeAlias = Annotated[Union[TrackObject, EpisodeObject], PropertyInfo(discriminator="type")]

Queue: TypeAlias = Annotated[Union[TrackObject, EpisodeObject], PropertyInfo(discriminator="type")]


class QueueGetResponse(BaseModel):
    currently_playing: Optional[CurrentlyPlaying] = None
    """The currently playing track or episode. Can be `null`."""

    queue: Optional[List[Queue]] = None
    """The tracks or episodes in the queue. Can be empty."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .image_object import ImageObject
from .author_object import AuthorObject
from .narrator_object import NarratorObject
from .copyright_object import CopyrightObject
from .external_url_object import ExternalURLObject

__all__ = ["AudiobookBase"]


class AudiobookBase(BaseModel):
    id: str
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    audiobook.
    """

    authors: List[AuthorObject]
    """The author(s) for the audiobook."""

    available_markets: List[str]
    """
    A list of the countries in which the audiobook can be played, identified by
    their [ISO 3166-1 alpha-2](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)
    code.
    """

    copyrights: List[CopyrightObject]
    """The copyright statements of the audiobook."""

    description: str
    """A description of the audiobook.

    HTML tags are stripped away from this field, use `html_description` field in
    case HTML tags are needed.
    """

    explicit: bool
    """
    Whether or not the audiobook has explicit content (true = yes it does; false =
    no it does not OR unknown).
    """

    external_urls: ExternalURLObject
    """External URLs for this audiobook."""

    href: str
    """A link to the Web API endpoint providing full details of the audiobook."""

    html_description: str
    """A description of the audiobook. This field may contain HTML tags."""

    images: List[ImageObject]
    """The cover art for the audiobook in various sizes, widest first."""

    languages: List[str]
    """
    A list of the languages used in the audiobook, identified by their
    [ISO 639](https://en.wikipedia.org/wiki/ISO_639) code.
    """

    media_type: str
    """The media type of the audiobook."""

    name: str
    """The name of the audiobook."""

    narrators: List[NarratorObject]
    """The narrator(s) for the audiobook."""

    publisher: str
    """The publisher of the audiobook."""

    total_chapters: int
    """The number of chapters in this audiobook."""

    type: Literal["audiobook"]
    """The object type."""

    uri: str
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    audiobook.
    """

    edition: Optional[str] = None
    """The edition of the audiobook."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..shared.paging_playlist_object import PagingPlaylistObject

__all__ = ["CategoryGetPlaylistsResponse"]


class CategoryGetPlaylistsResponse(BaseModel):
    message: Optional[str] = None
    """The localized message of a playlist."""

    playlists: Optional[PagingPlaylistObject] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["AlbumSaveParams"]


class AlbumSaveParams(TypedDict, total=False):
    ids: SequenceNotStr[str]
    """
    A JSON array of the
    [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
    `["4iV5W9uYEdYUVa79Axb7Rh", "1301WleyT98MSxVHPZCA6M"]`<br/>A maximum of 50 items
    can be specified in one request. _**Note**: if the `ids` parameter is present in
    the query string, any IDs listed here in the body will be ignored._
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.me.player import queue_add_params
from ....types.me.player.queue_get_response import QueueGetResponse

__all__ = ["QueueResource", "AsyncQueueResource"]


class QueueResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> QueueResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return QueueResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QueueResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return QueueResourceWithStreamingResponse(self)

    def add(
        self,
        *,
        uri: str,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Add an item to be played next in the user's current playback queue.

        This API
        only works for users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
          uri: The uri of the item to add to the queue. Must be a track or an episode uri.

          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/me/player/queue",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "uri": uri,
                        "device_id": device_id,
                    },
                    queue_add_params.QueueAddParams,
                ),
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueGetResponse:
        """Get the list of objects that make up the user's queue."""
        return self._get(
            "/me/player/queue",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueGetResponse,
        )


class AsyncQueueResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncQueueResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncQueueResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQueueResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncQueueResourceWithStreamingResponse(self)

    async def add(
        self,
        *,
        uri: str,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Add an item to be played next in the user's current playback queue.

        This API
        only works for users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
          uri: The uri of the item to add to the queue. Must be a track or an episode uri.

          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/me/player/queue",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "uri": uri,
                        "device_id": device_id,
                    },
                    queue_add_params.QueueAddParams,
                ),
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueGetResponse:
        """Get the list of objects that make up the user's queue."""
        return await self._get(
            "/me/player/queue",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueGetResponse,
        )


class QueueResourceWithRawResponse:
    def __init__(self, queue: QueueResource) -> None:
        self._queue = queue

        self.add = to_raw_response_wrapper(
            queue.add,
        )
        self.get = to_raw_response_wrapper(
            queue.get,
        )


class AsyncQueueResourceWithRawResponse:
    def __init__(self, queue: AsyncQueueResource) -> None:
        self._queue = queue

        self.add = async_to_raw_response_wrapper(
            queue.add,
        )
        self.get = async_to_raw_response_wrapper(
            queue.get,
        )


class QueueResourceWithStreamingResponse:
    def __init__(self, queue: QueueResource) -> None:
        self._queue = queue

        self.add = to_streamed_response_wrapper(
            queue.add,
        )
        self.get = to_streamed_response_wrapper(
            queue.get,
        )


class AsyncQueueResourceWithStreamingResponse:
    def __init__(self, queue: AsyncQueueResource) -> None:
        self._queue = queue

        self.add = async_to_streamed_response_wrapper(
            queue.add,
        )
        self.get = async_to_streamed_response_wrapper(
            queue.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["SearchQueryParams"]


class SearchQueryParams(TypedDict, total=False):
    q: Required[str]
    """Your search query.

    You can narrow down your search using field filters. The available filters are
    `album`, `artist`, `track`, `year`, `upc`, `tag:hipster`, `tag:new`, `isrc`, and
    `genre`. Each field filter only applies to certain result types.

    The `artist` and `year` filters can be used while searching albums, artists and
    tracks. You can filter on a single `year` or a range (e.g. 1955-1960).<br /> The
    `album` filter can be used while searching albums and tracks.<br /> The `genre`
    filter can be used while searching artists and tracks.<br /> The `isrc` and
    `track` filters can be used while searching tracks.<br /> The `upc`, `tag:new`
    and `tag:hipster` filters can only be used while searching albums. The `tag:new`
    filter will return albums released in the past two weeks and `tag:hipster` can
    be used to return only albums with the lowest 10% popularity.<br />
    """

    type: Required[List[Literal["album", "artist", "playlist", "track", "show", "episode", "audiobook"]]]
    """A comma-separated list of item types to search across.

    Search results include hits from all the specified item types. For example:
    `q=abacab&type=album,track` returns both albums and tracks matching "abacab".
    """

    include_external: Literal["audio"]
    """
    If `include_external=audio` is specified it signals that the client can play
    externally hosted audio content, and marks the content as playable in the
    response. By default externally hosted audio content is marked as unplayable in
    the response.
    """

    limit: int
    """The maximum number of results to return in each item type."""

    market: str
    """
    An
    [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    If a country code is specified, only content that is available in that market
    will be returned.<br/> If a valid user access token is specified in the request
    header, the country associated with the user account will take priority over
    this parameter.<br/> _**Note**: If neither market or user country are provided,
    the content is considered unavailable for the client._<br/> Users can view the
    country that is associated with their account in the
    [account settings](https://www.spotify.com/account/overview/).
    """

    offset: int
    """The index of the first result to return.

    Use with limit to get the next page of search results.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .time_interval_object import TimeIntervalObject

__all__ = ["AudioAnalysisRetrieveResponse", "Meta", "Section", "Segment", "Track"]


class Meta(BaseModel):
    analysis_time: Optional[float] = None
    """The amount of time taken to analyze this track."""

    analyzer_version: Optional[str] = None
    """The version of the Analyzer used to analyze this track."""

    detailed_status: Optional[str] = None
    """A detailed status code for this track.

    If analysis data is missing, this code may explain why.
    """

    input_process: Optional[str] = None
    """The method used to read the track's audio data."""

    platform: Optional[str] = None
    """The platform used to read the track's audio data."""

    status_code: Optional[int] = None
    """The return code of the analyzer process.

    0 if successful, 1 if any errors occurred.
    """

    timestamp: Optional[int] = None
    """The Unix timestamp (in seconds) at which this track was analyzed."""


class Section(BaseModel):
    confidence: Optional[float] = None
    """
    The confidence, from 0.0 to 1.0, of the reliability of the section's
    "designation".
    """

    duration: Optional[float] = None
    """The duration (in seconds) of the section."""

    key: Optional[int] = None
    """The estimated overall key of the section.

    The values in this field ranging from 0 to 11 mapping to pitches using standard
    Pitch Class notation (E.g. 0 = C, 1 = C♯/D♭, 2 = D, and so on). If no key was
    detected, the value is -1.
    """

    key_confidence: Optional[float] = None
    """The confidence, from 0.0 to 1.0, of the reliability of the key.

    Songs with many key changes may correspond to low values in this field.
    """

    loudness: Optional[float] = None
    """The overall loudness of the section in decibels (dB).

    Loudness values are useful for comparing relative loudness of sections within
    tracks.
    """

    mode: Optional[Literal[-1, 0, 1]] = None
    """
    Indicates the modality (major or minor) of a section, the type of scale from
    which its melodic content is derived. This field will contain a 0 for "minor", a
    1 for "major", or a -1 for no result. Note that the major key (e.g. C major)
    could more likely be confused with the minor key at 3 semitones lower (e.g. A
    minor) as both keys carry the same pitches.
    """

    mode_confidence: Optional[float] = None
    """The confidence, from 0.0 to 1.0, of the reliability of the `mode`."""

    start: Optional[float] = None
    """The starting point (in seconds) of the section."""

    tempo: Optional[float] = None
    """The overall estimated tempo of the section in beats per minute (BPM).

    In musical terminology, tempo is the speed or pace of a given piece and derives
    directly from the average beat duration.
    """

    tempo_confidence: Optional[float] = None
    """The confidence, from 0.0 to 1.0, of the reliability of the tempo.

    Some tracks contain tempo changes or sounds which don't contain tempo (like pure
    speech) which would correspond to a low value in this field.
    """

    time_signature: Optional[int] = None
    """An estimated time signature.

    The time signature (meter) is a notational convention to specify how many beats
    are in each bar (or measure). The time signature ranges from 3 to 7 indicating
    time signatures of "3/4", to "7/4".
    """

    time_signature_confidence: Optional[float] = None
    """The confidence, from 0.0 to 1.0, of the reliability of the `time_signature`.

    Sections with time signature changes may correspond to low values in this field.
    """


class Segment(BaseModel):
    confidence: Optional[float] = None
    """The confidence, from 0.0 to 1.0, of the reliability of the segmentation.

    Segments of the song which are difficult to logically segment (e.g: noise) may
    correspond to low values in this field.
    """

    duration: Optional[float] = None
    """The duration (in seconds) of the segment."""

    loudness_end: Optional[float] = None
    """The offset loudness of the segment in decibels (dB).

    This value should be equivalent to the loudness_start of the following segment.
    """

    loudness_max: Optional[float] = None
    """The peak loudness of the segment in decibels (dB).

    Combined with `loudness_start` and `loudness_max_time`, these components can be
    used to describe the "attack" of the segment.
    """

    loudness_max_time: Optional[float] = None
    """The segment-relative offset of the segment peak loudness in seconds.

    Combined with `loudness_start` and `loudness_max`, these components can be used
    to desctibe the "attack" of the segment.
    """

    loudness_start: Optional[float] = None
    """The onset loudness of the segment in decibels (dB).

    Combined with `loudness_max` and `loudness_max_time`, these components can be
    used to describe the "attack" of the segment.
    """

    pitches: Optional[List[float]] = None
    """
    Pitch content is given by a “chroma” vector, corresponding to the 12 pitch
    classes C, C#, D to B, with values ranging from 0 to 1 that describe the
    relative dominance of every pitch in the chromatic scale. For example a C Major
    chord would likely be represented by large values of C, E and G (i.e. classes 0,
    4, and 7).

    Vectors are normalized to 1 by their strongest dimension, therefore noisy sounds
    are likely represented by values that are all close to 1, while pure tones are
    described by one value at 1 (the pitch) and others near 0. As can be seen below,
    the 12 vector indices are a combination of low-power spectrum values at their
    respective pitch frequencies. ![pitch vector](/assets/audio/Pitch_vector.png)
    """

    start: Optional[float] = None
    """The starting point (in seconds) of the segment."""

    timbre: Optional[List[float]] = None
    """
    Timbre is the quality of a musical note or sound that distinguishes different
    types of musical instruments, or voices. It is a complex notion also referred to
    as sound color, texture, or tone quality, and is derived from the shape of a
    segment’s spectro-temporal surface, independently of pitch and loudness. The
    timbre feature is a vector that includes 12 unbounded values roughly centered
    around 0. Those values are high level abstractions of the spectral surface,
    ordered by degree of importance.

    For completeness however, the first dimension represents the average loudness of
    the segment; second emphasizes brightness; third is more closely correlated to
    the flatness of a sound; fourth to sounds with a stronger attack; etc. See an
    image below representing the 12 basis functions (i.e. template segments).
    ![timbre basis functions](/assets/audio/Timbre_basis_functions.png)

    The actual timbre of the segment is best described as a linear combination of
    these 12 basis functions weighted by the coefficient values: timbre = c1 x b1 +
    c2 x b2 + ... + c12 x b12, where c1 to c12 represent the 12 coefficients and b1
    to b12 the 12 basis functions as displayed below. Timbre vectors are best used
    in comparison with each other.
    """


class Track(BaseModel):
    analysis_channels: Optional[int] = None
    """The number of channels used for analysis.

    If 1, all channels are summed together to mono before analysis.
    """

    analysis_sample_rate: Optional[int] = None
    """The sample rate used to decode and analyze this track.

    May differ from the actual sample rate of this track available on Spotify.
    """

    code_version: Optional[float] = None
    """
    A version number for the Echo Nest Musical Fingerprint format used in the
    codestring field.
    """

    codestring: Optional[str] = None
    """
    An
    [Echo Nest Musical Fingerprint (ENMFP)](https://academiccommons.columbia.edu/doi/10.7916/D8Q248M4)
    codestring for this track.
    """

    duration: Optional[float] = None
    """Length of the track in seconds."""

    echoprint_version: Optional[float] = None
    """A version number for the EchoPrint format used in the echoprintstring field."""

    echoprintstring: Optional[str] = None
    """
    An [EchoPrint](https://github.com/spotify/echoprint-codegen) codestring for this
    track.
    """

    end_of_fade_in: Optional[float] = None
    """The time, in seconds, at which the track's fade-in period ends.

    If the track has no fade-in, this will be 0.0.
    """

    key: Optional[int] = None
    """The key the track is in.

    Integers map to pitches using standard
    [Pitch Class notation](https://en.wikipedia.org/wiki/Pitch_class). E.g. 0 = C, 1
    = C♯/D♭, 2 = D, and so on. If no key was detected, the value is -1.
    """

    key_confidence: Optional[float] = None
    """The confidence, from 0.0 to 1.0, of the reliability of the `key`."""

    loudness: Optional[float] = None
    """The overall loudness of a track in decibels (dB).

    Loudness values are averaged across the entire track and are useful for
    comparing relative loudness of tracks. Loudness is the quality of a sound that
    is the primary psychological correlate of physical strength (amplitude). Values
    typically range between -60 and 0 db.
    """

    mode: Optional[int] = None
    """
    Mode indicates the modality (major or minor) of a track, the type of scale from
    which its melodic content is derived. Major is represented by 1 and minor is 0.
    """

    mode_confidence: Optional[float] = None
    """The confidence, from 0.0 to 1.0, of the reliability of the `mode`."""

    num_samples: Optional[int] = None
    """The exact number of audio samples analyzed from this track.

    See also `analysis_sample_rate`.
    """

    offset_seconds: Optional[int] = None
    """An offset to the start of the region of the track that was analyzed.

    (As the entire track is analyzed, this should always be 0.)
    """

    rhythm_version: Optional[float] = None
    """A version number for the Rhythmstring used in the rhythmstring field."""

    rhythmstring: Optional[str] = None
    """A Rhythmstring for this track.

    The format of this string is similar to the Synchstring.
    """

    sample_md5: Optional[str] = None
    """This field will always contain the empty string."""

    start_of_fade_out: Optional[float] = None
    """The time, in seconds, at which the track's fade-out period starts.

    If the track has no fade-out, this should match the track's length.
    """

    synch_version: Optional[float] = None
    """A version number for the Synchstring used in the synchstring field."""

    synchstring: Optional[str] = None
    """A [Synchstring](https://github.com/echonest/synchdata) for this track."""

    tempo: Optional[float] = None
    """The overall estimated tempo of a track in beats per minute (BPM).

    In musical terminology, tempo is the speed or pace of a given piece and derives
    directly from the average beat duration.
    """

    tempo_confidence: Optional[float] = None
    """The confidence, from 0.0 to 1.0, of the reliability of the `tempo`."""

    time_signature: Optional[int] = None
    """An estimated time signature.

    The time signature (meter) is a notational convention to specify how many beats
    are in each bar (or measure). The time signature ranges from 3 to 7 indicating
    time signatures of "3/4", to "7/4".
    """

    time_signature_confidence: Optional[float] = None
    """The confidence, from 0.0 to 1.0, of the reliability of the `time_signature`."""

    window_seconds: Optional[int] = None
    """
    The length of the region of the track was analyzed, if a subset of the track was
    analyzed. (As the entire track is analyzed, this should always be 0.)
    """


class AudioAnalysisRetrieveResponse(BaseModel):
    bars: Optional[List[TimeIntervalObject]] = None
    """The time intervals of the bars throughout the track.

    A bar (or measure) is a segment of time defined as a given number of beats.
    """

    beats: Optional[List[TimeIntervalObject]] = None
    """The time intervals of beats throughout the track.

    A beat is the basic time unit of a piece of music; for example, each tick of a
    metronome. Beats are typically multiples of tatums.
    """

    meta: Optional[Meta] = None

    sections: Optional[List[Section]] = None
    """Sections are defined by large variations in rhythm or timbre, e.g.

    chorus, verse, bridge, guitar solo, etc. Each section contains its own
    descriptions of tempo, key, mode, time_signature, and loudness.
    """

    segments: Optional[List[Segment]] = None
    """Each segment contains a roughly conisistent sound throughout its duration."""

    tatums: Optional[List[TimeIntervalObject]] = None
    """
    A tatum represents the lowest regular pulse train that a listener intuitively
    infers from the timing of perceived musical events (segments).
    """

    track: Optional[Track] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PlayerSetRepeatModeParams"]


class PlayerSetRepeatModeParams(TypedDict, total=False):
    state: Required[str]
    """
    **track**, **context** or **off**.<br/> **track** will repeat the current
    track.<br/> **context** will repeat the current context.<br/> **off** will turn
    repeat off.
    """

    device_id: str
    """The id of the device this command is targeting.

    If not supplied, the user's currently active device is the target.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorURLPage, AsyncCursorURLPage
from ...types.users import playlist_list_params, playlist_create_params
from ..._base_client import AsyncPaginator, make_request_options
from ...types.users.playlist_create_response import PlaylistCreateResponse
from ...types.shared.simplified_playlist_object import SimplifiedPlaylistObject

__all__ = ["PlaylistsResource", "AsyncPlaylistsResource"]


class PlaylistsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PlaylistsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return PlaylistsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlaylistsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return PlaylistsResourceWithStreamingResponse(self)

    def create(
        self,
        user_id: str,
        *,
        name: str,
        collaborative: bool | Omit = omit,
        description: str | Omit = omit,
        public: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlaylistCreateResponse:
        """Create a playlist for a Spotify user.

        (The playlist will be empty until you
        [add tracks](/documentation/web-api/reference/add-tracks-to-playlist).) Each
        user is generally limited to a maximum of 11000 playlists.

        Args:
          user_id: The user's [Spotify user ID](/documentation/web-api/concepts/spotify-uris-ids).

          name: The name for the new playlist, for example `"Your Coolest Playlist"`. This name
              does not need to be unique; a user may have several playlists with the same
              name.

          collaborative: Defaults to `false`. If `true` the playlist will be collaborative. _**Note**: to
              create a collaborative playlist you must also set `public` to `false`. To create
              collaborative playlists you must have granted `playlist-modify-private` and
              `playlist-modify-public`
              [scopes](/documentation/web-api/concepts/scopes/#list-of-scopes)._

          description: value for playlist description as displayed in Spotify Clients and in the Web
              API.

          public: Defaults to `true`. The playlist's public/private status (if it should be added
              to the user's profile or not): `true` the playlist will be public, `false` the
              playlist will be private. To be able to create private playlists, the user must
              have granted the `playlist-modify-private`
              [scope](/documentation/web-api/concepts/scopes/#list-of-scopes). For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._post(
            f"/users/{user_id}/playlists",
            body=maybe_transform(
                {
                    "name": name,
                    "collaborative": collaborative,
                    "description": description,
                    "public": public,
                },
                playlist_create_params.PlaylistCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlaylistCreateResponse,
        )

    def list(
        self,
        user_id: str,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[SimplifiedPlaylistObject]:
        """
        Get a list of the playlists owned or followed by a Spotify user.

        Args:
          user_id: The user's [Spotify user ID](/documentation/web-api/concepts/spotify-uris-ids).

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first playlist to return. Default: 0 (the first object).
              Maximum offset: 100.000\\.. Use with `limit` to get the next set of playlists.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get_api_list(
            f"/users/{user_id}/playlists",
            page=SyncCursorURLPage[SimplifiedPlaylistObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    playlist_list_params.PlaylistListParams,
                ),
            ),
            model=SimplifiedPlaylistObject,
        )


class AsyncPlaylistsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPlaylistsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncPlaylistsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlaylistsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncPlaylistsResourceWithStreamingResponse(self)

    async def create(
        self,
        user_id: str,
        *,
        name: str,
        collaborative: bool | Omit = omit,
        description: str | Omit = omit,
        public: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlaylistCreateResponse:
        """Create a playlist for a Spotify user.

        (The playlist will be empty until you
        [add tracks](/documentation/web-api/reference/add-tracks-to-playlist).) Each
        user is generally limited to a maximum of 11000 playlists.

        Args:
          user_id: The user's [Spotify user ID](/documentation/web-api/concepts/spotify-uris-ids).

          name: The name for the new playlist, for example `"Your Coolest Playlist"`. This name
              does not need to be unique; a user may have several playlists with the same
              name.

          collaborative: Defaults to `false`. If `true` the playlist will be collaborative. _**Note**: to
              create a collaborative playlist you must also set `public` to `false`. To create
              collaborative playlists you must have granted `playlist-modify-private` and
              `playlist-modify-public`
              [scopes](/documentation/web-api/concepts/scopes/#list-of-scopes)._

          description: value for playlist description as displayed in Spotify Clients and in the Web
              API.

          public: Defaults to `true`. The playlist's public/private status (if it should be added
              to the user's profile or not): `true` the playlist will be public, `false` the
              playlist will be private. To be able to create private playlists, the user must
              have granted the `playlist-modify-private`
              [scope](/documentation/web-api/concepts/scopes/#list-of-scopes). For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._post(
            f"/users/{user_id}/playlists",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "collaborative": collaborative,
                    "description": description,
                    "public": public,
                },
                playlist_create_params.PlaylistCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlaylistCreateResponse,
        )

    def list(
        self,
        user_id: str,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SimplifiedPlaylistObject, AsyncCursorURLPage[SimplifiedPlaylistObject]]:
        """
        Get a list of the playlists owned or followed by a Spotify user.

        Args:
          user_id: The user's [Spotify user ID](/documentation/web-api/concepts/spotify-uris-ids).

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first playlist to return. Default: 0 (the first object).
              Maximum offset: 100.000\\.. Use with `limit` to get the next set of playlists.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get_api_list(
            f"/users/{user_id}/playlists",
            page=AsyncCursorURLPage[SimplifiedPlaylistObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    playlist_list_params.PlaylistListParams,
                ),
            ),
            model=SimplifiedPlaylistObject,
        )


class PlaylistsResourceWithRawResponse:
    def __init__(self, playlists: PlaylistsResource) -> None:
        self._playlists = playlists

        self.create = to_raw_response_wrapper(
            playlists.create,
        )
        self.list = to_raw_response_wrapper(
            playlists.list,
        )


class AsyncPlaylistsResourceWithRawResponse:
    def __init__(self, playlists: AsyncPlaylistsResource) -> None:
        self._playlists = playlists

        self.create = async_to_raw_response_wrapper(
            playlists.create,
        )
        self.list = async_to_raw_response_wrapper(
            playlists.list,
        )


class PlaylistsResourceWithStreamingResponse:
    def __init__(self, playlists: PlaylistsResource) -> None:
        self._playlists = playlists

        self.create = to_streamed_response_wrapper(
            playlists.create,
        )
        self.list = to_streamed_response_wrapper(
            playlists.list,
        )


class AsyncPlaylistsResourceWithStreamingResponse:
    def __init__(self, playlists: AsyncPlaylistsResource) -> None:
        self._playlists = playlists

        self.create = async_to_streamed_response_wrapper(
            playlists.create,
        )
        self.list = async_to_streamed_response_wrapper(
            playlists.list,
        )
